CREATE OR REPLACE PACKAGE Primes
AS
   ---------------------------------------------------------------
   -- Package Primes: simple prime number facilities
   --
   -- Copyright (c) 2002 by John M Beresniewicz, all rights reserved.
   -- 
   -- This package originally developed to demonstrate application
   -- of Design by Contract to PL/SQL programming.
   ---------------------------------------------------------------

   -------------------------------------------------------------------
   -- index-by tables of integers are the basic structures manipulated
   --
   TYPE Primes_tbltype IS TABLE OF INTEGER INDEX BY BINARY_INTEGER;
   SUBTYPE factors_tbltype IS Primes_tbltype;

   -------------------------------------------------------------------
   -- exception for data beyond scope of package
   -- (thrown by accept for input validation by clients)
   OUT_OF_RANGE      EXCEPTION;
   OUT_OF_RANGE_C    CONSTANT INTEGER := -20111;
   PRAGMA EXCEPTION_INIT(OUT_OF_RANGE, -20111);

   ----------------------------------------------------------
   --  Supplied modules (features) of the package
   --
   --  NextP returns next prime larger than input
   --  PrevP returns first prime less than input
   --  isPrime returns TRUE for prime input
   --  Factors returns table of prime factors for input
   --  txtFactors returns varchar serialization of Factors
   --  accept procedure validates client input or 
   --                   raises out_of_range
   --
   FUNCTION NextP (num_IN INTEGER) RETURN INTEGER;
   FUNCTION PrevP (num_IN INTEGER) RETURN INTEGER;
   FUNCTION isPrime (num_IN INTEGER) RETURN BOOLEAN;

   FUNCTION Factors(num_IN INTEGER) RETURN factors_tbltype;
   FUNCTION txtFactors(num_IN INTEGER) RETURN VARCHAR2;

   PROCEDURE accept(num_IN INTEGER);

   ----------------------------------------------------------
   -- public SLPA declarations (standard local package assertion)
   --
   ASSERTFAIL     EXCEPTION;
   PRAGMA EXCEPTION_INIT(ASSERTFAIL, -20999);
   ----------------------------------------------------------
   -- privatize assert for production to preclude coupling 
   -- PROCEDURE assert (bool_IN IN BOOLEAN, msg_IN IN VARCHAR2 := null);
END Primes;
/



select * from user_errors;

